@@ -7,6 +7,7 @@ use XML::LibXML; #::Element;
 use XML::LibXML::NodeList;
 use XBRL::Arc;
 use HTML::Table;
+use XBRL::TableXML;
 
 require Exporter;
 
@@ -22,7 +23,7 @@ our @EXPORT = qw(
 
 );
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 sub new() {
 	my ($class, $xbrl_doc, $uri) = @_;
@@ -30,12 +31,10 @@ sub new() {
 								uri => $uri };
 	bless $self, $class;
 	
-	$self->{'def_arcs'} = &parse_def_arcs($self);	
-	$self->{'pre_arcs'} = &parse_pre_arcs($self);	
 	return $self;
 }
 
-sub get_html_table() {
+sub get_xml_table() {
 	my ($self) = @_;
 	my $table;
 	if (&is_landscape($self)) {
@@ -54,7 +53,8 @@ sub make_port_table() {
 	my $uri = $self->{'uri'};	
 	my $xbrl_doc = $self->{'xbrl'};
 	my $tax = $xbrl_doc->get_taxonomy();
-	my $table = HTML::Table->new(-border => 1);
+#	my $table = HTML::Table->new(-border => 1);
+	my $table = XBRL::TableXML->new(); 
 	
 	my $header_contexts	= &get_header_contexts($self); 
 	my @col_labels;
@@ -63,7 +63,9 @@ sub make_port_table() {
 	for my $context (@{$header_contexts}) {
 		push(@col_labels, $context->label());	
 	}
-	$table->addRow('&nbsp;', @col_labels); 	
+	#TODO find millions or thousands field and use it in
+	#the first column
+	$table->addHeader('&nbsp;', @col_labels); 	
 
 	#my (@domain_names, @row_elements);	
 	for my $hcube (@{$hypercubes}) {
@@ -118,7 +120,8 @@ sub make_port_table() {
 	
 	&set_row_labels($self, $table, $uri);	
 
-	return $table->getTable();
+	#return $table->as_text();
+	return $table;
 }
 
 
@@ -139,7 +142,8 @@ sub make_land_table() {
 		push(@col_elements, @{$tmp_cols});	
 	}	
 
-	$table->addRow('&nbsp;', @col_elements);
+	#$table->addRow('&nbsp;', @col_elements);
+	$table->addHeader('&nbsp;', @col_elements); 	
 
 	for my $e (@row_elements) {
 		$table->addRow($e);	
@@ -178,7 +182,8 @@ sub make_land_table() {
 		$table->setCell(1, $i, $label);	
 	}
 
-	return $table->getTable();
+	#return $table->getTable();
+	return $table;
 }
 
 sub set_row_labels() {
@@ -186,17 +191,22 @@ sub set_row_labels() {
 	my $uri = $self->{'uri'};	
 	my $xbrl_doc = $self->{'xbrl'};
 	my $tax = $xbrl_doc->get_taxonomy();
-	my @p_arcs = @{$self->{'pre_arcs'}}; 
+#	my @p_arcs = @{$self->{'pre_arcs'}}; 
+	my $p_arcs = $tax->get_arcs('pre', $uri);
+
+
 	#TODO Deal with different preferred labels for the same id in the same table
 	#this code just takes the first one for every instance.
 	for (my $i = 1; $i <= $table->getTableRows; $i++) {
-		my $id = $table->getCell($i,1);
+		#my $id = $table->getCell($i,1);
+		my $id = $table->label($i); 	
 		$id =~ s/\:/\_/;	
-		for (my $k = 0; $k < @p_arcs; $k++) {
-				if ($id eq $p_arcs[$k]->to_short()) {
+		for (my $k = 0; $k < @{$p_arcs}; $k++) {
+				if ($id eq $p_arcs->[$k]->to_short()) {
 				#Get the label delete the entry from the the array
-					my $label = $tax->get_label($id, $p_arcs[$k]->prefLabel());				
-					$table->setCell($i, 1, $label);	
+					my $label = $tax->get_label($id, $p_arcs->[$k]->prefLabel());				
+					$table->label($i, $label);	
+					#$table->setCell($i, 1, $label);	
 					#delete $p_arcs[$k];	
 			}
 		}
@@ -391,7 +401,10 @@ sub get_domain_names() {
 	#take the uri and return an array of col elements + names in anon hash 	
 	#for landscape dimension tables  
 	my ($self, $hcube) = @_;
-	my $arcs = $self->{'def_arcs'};
+	my $xbrl_tax = $self->{'xbrl'}->get_taxonomy();
+	my $uri = $self->{'uri'};	
+	#my ($self, $type, $uri) = @_;
+	my $arcs = $xbrl_tax->get_arcs('def', $uri );   
 	my @top_domains;
 	my @sub_domains;
 	my @middle_domains;
@@ -460,8 +473,14 @@ sub get_row_elements() {
 	#take a uri and return an array of anonymous hash with element id + pref label
 	#for landscape dimension tables 	
 	my ($self, $hcube) = @_;
-	my $arcs = $self->{'def_arcs'};
-		my $arc_all;	
+	#my $arcs = $self->{'def_arcs'};
+	my $xbrl_tax = $self->{'xbrl'}->get_taxonomy();
+	my $uri = $self->{'uri'};	
+	#my ($self, $type, $uri) = @_;
+	my $arcs = $xbrl_tax->get_arcs('def', $uri );   
+	
+	
+	my $arc_all;	
 		my $dimension_default;
 		my $dimension_domain;	
 		
@@ -508,20 +527,21 @@ sub is_landscape() {
 	my $def_uri = $self->{'uri'};	
 	my $xbrl_doc = $self->{'xbrl'};	
 	my $tax = $xbrl_doc->get_taxonomy();
-	my $preLB = $tax->pre();
+#	my $preLB = $tax->pre();
+	my $start = 'http://www.xbrl.org/2003/role/periodStartLabel'; 
+	my $end = 'http://www.xbrl.org/2003/role/periodEndLabel'; 
 
-	my $p_link = $preLB->findnodes("//*[local-name() = 'presentationLink'][\@xlink:role = '" . $def_uri . "' ]"); 
-	
-	my $preArcs = $p_link->[0]->getChildrenByLocalName('presentationArc');
-	
-	for my $arc (@{$preArcs}) {
-		my $label = $arc->getAttribute('preferredLabel');
-		next unless ($label);
-		if (($label eq  'http://www.xbrl.org/2003/role/periodStartLabel') or 
-		($label eq  'http://www.xbrl.org/2003/role/periodEndLabel') ) {
-			return "true";
+	my $arcs = $tax->get_arcs('pre', $def_uri);
+
+	for my $arc (@{$arcs}) {
+		if ($arc->prefLabel) {
+		 	if (($arc->prefLabel eq $start) or ($arc->prefLabel eq $end) ) {
+				return 'true';
+			}
 		}
 	}
+
+
 }
 
 
@@ -530,8 +550,14 @@ sub is_landscape() {
 sub get_header_contexts() {
 	my ($self) = @_;
 	my $xbrl_doc = $self->{'xbrl'};
+	
+	my $xbrl_tax = $self->{'xbrl'}->get_taxonomy();
+	my $uri = $self->{'uri'};	
+	#my ($self, $type, $uri) = @_;
+	my $arcs = $xbrl_tax->get_arcs('def', $uri );   
+	
 
-	my $arcs = $self->{'def_arcs'}; 
+#	my $arcs = $self->{'def_arcs'}; 
 	
 	my $all_contexts = $xbrl_doc->get_all_contexts();	
 
@@ -640,105 +666,14 @@ sub get_def_section() {
 
 
 
-sub parse_pre_arcs() {
-	my ($self) = @_;
-	my @out_array;	
-	my $xbrl_doc = $self->{'xbrl'};
-	my $uri = $self->{'uri'};
-	my $tax = $xbrl_doc->get_taxonomy();
-	my $preLB = $tax->pre(); 
-	
-	my $p_link = $preLB->findnodes("//*[local-name() = 'presentationLink'][\@xlink:role = '" . $uri . "' ]"); 
-	
-	unless ($p_link) {return undef; }
-
-	my @loc_links = $p_link->[0]->getChildrenByLocalName('loc'); 
-	my @arc_links = $p_link->[0]->getChildrenByLocalName('presentationArc'); 
-
-	for my $arc_xml (@arc_links) {
-		my $arc = XBRL::Arc->new();
-		$arc->order($arc_xml->getAttribute('order'));	
-		$arc->arcrole($arc_xml->getAttribute('xlink:arcrole'));
-		$arc->prefLabel($arc_xml->getAttribute('preferredLabel')); 	
-		
-		for my $loc_xml (@loc_links) {
-				
-			if ($loc_xml->getAttribute('xlink:label') eq $arc_xml->getAttribute('xlink:to')) {
-				#This is the destination loc link
-				my $href = $loc_xml->getAttribute('xlink:href');	
-				$arc->to_full($href);				
-				$href =~ m/\#([A-Za-z0-9_-].+)$/; 	
-				$arc->to_short($1);	
-			
-			}
-			elsif ($loc_xml->getAttribute('xlink:label') eq $arc_xml->getAttribute('xlink:from')  ) {
-				#this is the from link
-				my $href = $loc_xml->getAttribute('xlink:href');	
-				$arc->from_full($href);				
-				$href =~ m/\#([A-Za-z0-9_-].+)$/; 	
-				$arc->from_short($1);	
-			
-			}
-
-		}
-		push(@out_array, $arc);
-	}
-
-	return \@out_array;
-}
-
-
-sub parse_def_arcs() {
-	my ($self) = @_;
-	my @out_array;	
-	my $xbrl_doc = $self->{'xbrl'};
-	my $uri = $self->{'uri'};
-	my $tax = $xbrl_doc->get_taxonomy();
-	my $defLB = $tax->def(); 
-	my $d_link = $defLB->findnodes("//*[local-name() = 'definitionLink'][\@xlink:role = '" . $uri . "' ]"); 
-	
-	unless ($d_link) {return undef; }
-
-	my @loc_links = $d_link->[0]->getChildrenByLocalName('loc'); 
-	my @arc_links = $d_link->[0]->getChildrenByLocalName('definitionArc'); 
-
-	for my $arc_xml (@arc_links) {
-		my $arc = XBRL::Arc->new();
-		$arc->order($arc_xml->getAttribute('order'));	
-		$arc->arcrole($arc_xml->getAttribute('xlink:arcrole'));
-		$arc->closed($arc_xml->getAttribute('xbrldt:closed'));
-		$arc->usable($arc_xml->getAttribute('xbrldt:usable'));
-		$arc->contextElement($arc_xml->getAttribute('xbrldt:contextElement'));
-	
-		for my $loc_xml (@loc_links) {
-				
-			if ($loc_xml->getAttribute('xlink:label') eq $arc_xml->getAttribute('xlink:to')) {
-				#This is the destination loc link
-				my $href = $loc_xml->getAttribute('xlink:href');	
-				$arc->to_full($href);				
-				$href =~ m/\#([A-Za-z0-9_-].+)$/; 	
-				$arc->to_short($1);	
-			
-			}
-			elsif ($loc_xml->getAttribute('xlink:label') eq $arc_xml->getAttribute('xlink:from')  ) {
-				#this is the from link
-				my $href = $loc_xml->getAttribute('xlink:href');	
-				$arc->from_full($href);				
-				$href =~ m/\#([A-Za-z0-9_-].+)$/; 	
-				$arc->from_short($1);	
-			
-			}
-
-		}
-		push(@out_array, $arc);
-	}
-
-	return \@out_array;
-}
-
 sub get_hypercubes() {
 	my ($self) = @_;
-	my $arcs = $self->{'def_arcs'};	
+	my $xbrl_tax = $self->{'xbrl'}->get_taxonomy();
+	my $uri = $self->{'uri'};	
+	#my ($self, $type, $uri) = @_;
+	my $arcs = $xbrl_tax->get_arcs('def', $uri );   
+	
+	#my $arcs = $self->{'def_arcs'};	
 	my @hypercubes;	
 				
 	for my $arc (@{$arcs}) {
@@ -761,17 +696,30 @@ XBRL::Dimension - OO Module for Parsing XBRL Dimensions
 	my $dimension = XBRL::Dimension->new($xbrl_doc, "http://fu.bar.com/role/DisclosureGoodwillDetails" );	
 	
 	
-	$html_table = $dimension->get_html_table(); 
+	$html_table = $dimension->get_xml_table(); 
   
 	
 =head1 DESCRIPTION
 
 This module is part of the XBRL modules group and is intended for use with XBRL.
 
-new() Object constructor takes the xbrl document and the section URI as parameters.
+=over 4
 
-get_html_table()  -- Returns a scalar containing an HTML representation of the 
+=item new 
+
+	my $dimension = XBRL::Dimension->new($xbrl_doc, "http://fu.bar.com/role/DisclosureGoodwillDetails" );	
+
+
+Object constructor takes the xbrl document and the section URI as parameters.
+
+=item get_xml_table
+
+	$xml_table = $dimension->get_xml_table(); 
+
+Returns a scalar containing an HTML representation of the 
 				
+=back
+
 =head1 AUTHOR
 
 Mark Gannon <mark@truenorth.nu>
